@@ -1,5 +1,6 @@
 /* argcv.c - simple functions for parsing input based on whitespace
-   Copyright (C) 1999, 2000, 2001 Free Software Foundation, Inc.
+   Copyright (C) 1999, 2000, 2001, 2003, 2004, 2005,
+   2007 Free Software Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -11,17 +12,30 @@
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.
 
-   You should have received a copy of the GNU Lesser General Public
-   License along with this library; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA  */
+   You should have received a copy of the GNU Lesser General
+   Public License along with this library; if not, write to the
+   Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02110-1301 USA */
 
 #ifdef HAVE_CONFIG_H
 # include <config.h>
 #endif
 
 #include <ctype.h>
+#include <errno.h>
 #include <argcv.h>
 
+#define argcv_get             grad_argcv_get
+#define argcv_string          grad_argcv_string
+#define argcv_free            grad_argcv_free
+#define argcv_unquote_char    grad_argcv_unquote_char
+#define argcv_quote_char      grad_argcv_quote_char 
+#define argcv_quoted_length   grad_argcv_quoted_length
+#define argcv_quoted_length_n grad_argcv_quoted_length_n
+#define argcv_unquote_copy    grad_argcv_unquote_copy
+#define argcv_quote_copy      grad_argcv_quote_copy    
+#define argcv_quote_copy_n    grad_argcv_quote_copy_n 
+
 /*
  * takes a string and splits it into several strings, breaking at ' '
  * command is the string to split
@@ -34,11 +48,10 @@
 #define isdelim(c,delim) (strchr(delim,(c))!=NULL)
 
 static int
-argcv_scan (int len, const char *command, const char *delim, const char* cmnt,
+argcv_scan (int len, const char *command, const char *delim, const char *cmnt,
 	    int *start, int *end, int *save)
 {
   int i = 0;
-  int expect_delim;
   
   for (;;)
     {
@@ -52,44 +65,35 @@ argcv_scan (int len, const char *command, const char *delim, const char* cmnt,
 	i++;
       *start = i;
 
-      switch (command[i])
+      if (!isdelim (command[i], delim))
 	{
-	case '"':
-	case '\'':
-	  while (++i < len
-		 && (command[i] != command[*start]
-		     || command[i-1] == '\\'))
-	    ;
-	  if (i < len)		/* found matching quote */
-	    break;
-	 /*FALLTHRU*/ default:
-	  if (isdelim (command[i], delim))
-	    break;
-	  /* Skip until next whitespace character or end of line. Honor
-	     escaped whitespace. */
-	  expect_delim = 0;
-	  while (++i < len)
+	  while (i < len)
 	    {
-	      if (expect_delim)
+	      if (command[i] == '\\')
 		{
-		  if (command[i-1] != '\\' && command[i] == expect_delim)
-		    expect_delim = 0;
-		  else
-		    continue;
+		  if (++i == len)
+		    break;
+		  i++;
+		  continue;
 		}
 	      
-	      if (command[i-1] != '\\')
+	      if (command[i] == '\'' || command[i] == '"')
 		{
-		  if (command[i] == '\'' || command[i] == '"')
-		    expect_delim = command[i];
-		  else if (isws (command[i]) || isdelim (command[i], delim))
-		    break;
+		  int j;
+		  for (j = i+1; j < len && command[j] != command[i]; j++)
+		    if (command[j] == '\\')
+		      j++;
+		  if (j < len)
+		    i = j+1;
+		  else
+		    i++;
 		}
+	      else if (isws (command[i]) || isdelim (command[i], delim))
+		break;
 	      else
 		i++; /* skip the escaped character */
 	    }
 	  i--;
-	  break;
 	}
 
       *end = i;
@@ -99,7 +103,7 @@ argcv_scan (int len, const char *command, const char *delim, const char* cmnt,
          to the newline and restart the token search. */
       if (*save <= len)
 	{
-	  if (cmnt && strchr (cmnt, command[*start]) != NULL)
+	  if (strchr (cmnt, command[*start]) != NULL)
 	    {
 	      i = *save;
 	      while (i < len && command[i] != '\n')
@@ -143,27 +147,33 @@ argcv_quote_char (int c)
   return -1;
 }
   
+#define to_num(c) \
+  (isdigit(c) ? c - '0' : (isxdigit(c) ? toupper(c) - 'A' + 10 : 255 ))
 
 static int
-xtonum (const char *src, int base, size_t cnt)
+xtonum (int *pval, const char *src, int base, int cnt)
 {
-  int val;
-  char *p;
-  char tmp[4]; /* At most three characters + zero */
+  int i, val;
   
-  /* Notice: No use to check `cnt'. It should be either 2 or 3 */
-  memcpy (tmp, src, cnt);
-  tmp[cnt] = 0;
-  val = strtoul (tmp, &p, base);
-  return (*p == 0) ? val : -1;
+  for (i = 0, val = 0; i < cnt; i++, src++)
+    {
+      int n = *(unsigned char*)src;
+      if (n > 127 || (n = to_num(n)) >= base)
+	break;
+      val = val*base + n;
+    }
+  *pval = val;
+  return i;
 }
 
 size_t
-argcv_quoted_length (const char *str, int *quote)
+argcv_quoted_length_n (const char *str, size_t size, int *quote)
 {
   size_t len = 0;
+  const char *end = str + size;
 
-  for (; *str; str++)
+  *quote = 0;
+  for (; str < end; str++)
     {
       if (*str == ' ')
 	{
@@ -175,7 +185,7 @@ argcv_quoted_length (const char *str, int *quote)
 	  len += 2;
 	  *quote = 1;
 	}
-      else if (isprint (*str))
+      else if (*str != '\t' && *str != '\\' && isprint (*str))
 	len++;
       else if (argcv_quote_char (*str) != -1)
 	len += 2;
@@ -185,104 +195,117 @@ argcv_quoted_length (const char *str, int *quote)
   return len;
 }
 
+size_t
+argcv_quoted_length (const char *str, int *quote)
+{
+	return argcv_quoted_length_n (str, strlen (str), quote);
+}
+
 void
 argcv_unquote_copy (char *dst, const char *src, size_t n)
 {
+  int i = 0;
   int c;
-  int expect_delim = 0;
-  
-  while (n > 0)
+  int expect_delim = 0; 
+    
+  while (i < n)
     {
-      n--;
-
-      switch (*src)
+      switch (src[i])
 	{
 	case '\'':
 	case '"':
-	  ++src;
-	  if (expect_delim)
-	    expect_delim = 0;
+	  if (!expect_delim)
+	    {
+	      const char *p;
+	      
+	      for (p = src+i+1; *p && *p != src[i]; p++)
+		if (*p == '\\')
+		  p++;
+	      if (*p)
+		expect_delim = src[i++];
+	      else
+		*dst++ = src[i++];
+	    }
+	  else if (expect_delim == src[i])
+	    ++i;
 	  else
-	    expect_delim = *src;
+	    *dst++ = src[i++];
 	  break;
 	  
 	case '\\':
-	  switch (*++src)
+	  ++i;
+	  if (src[i] == 0)
 	    {
-	    case 'x':
-	    case 'X':
-	      ++src;
-	      --n;
-	      if (n == 0)
-		{
-		  *dst++ = '\\';
-		  *dst++ = src[-1];
-		}
-	      else
-		{
-		  c = xtonum(src, 16, 2);
-		  if (c == -1)
-		    {
-		      *dst++ = '\\';
-		      *dst++ = src[-1];
-		    }
-		  else
-		    {
-		      *dst++ = c;
-		      src += 2;
-		      n -= 2;
-		    }
-		}
-	      break;
-	      
-	    case '0':
-	      ++src;
-	      --n;
-	      if (n == 0)
-		{
-		  *dst++ = '\\';
-		  *dst++ = src[-1];
-		}
-	      else
-		{
-		  c = xtonum(src, 8, 3);
-		  if (c == -1)
-		    {
-		      *dst++ = '\\';
-		      *dst++ = src[-1];
-		    }
-		  else
-		    {
-		      *dst++ = c;
-		      src += 3;
-		      n -= 3;
-		    }
-		}
-	      break;
-	      
-	    default:
-	      *dst++ = argcv_unquote_char (*src++);
-	      n--;
+	      *dst++ = '\\';
+	      *dst++ = src[i++];
 	    }
-
+	  else if (src[i] == 'x' || src[i] == 'X')
+	    {
+	      if (n - i < 2)
+		{
+		  *dst++ = '\\';
+		  *dst++ = src[i++];
+		}
+	      else 
+		{
+		  int off = xtonum(&c, src + i + 1, 16, 2);
+		  if (off == 0)
+		    {
+		      *dst++ = '\\';
+		      *dst++ = src[i++];
+		    }
+		  else
+		    {
+		      *dst++ = c;
+		      i += off + 1;
+		    }
+		}
+	    }
+	  else if ((unsigned char)src[i] < 128 && isdigit(src[i]))
+	    {
+	      if (n - i < 1)
+		{
+		  *dst++ = '\\';
+		  *dst++ = src[i++];
+		}
+	      else
+		{
+		  int off = xtonum(&c, src+i, 8, 3);
+		  if (off == 0)
+		    {
+		      *dst++ = '\\';
+		      *dst++ = src[i++];
+		    }
+		  else
+		    {
+		      *dst++ = c;
+		      i += off;
+		    }
+		}
+	    }
+	  else
+	    *dst++ = argcv_unquote_char (src[i++]);
+	  break;
+	  
 	default:
-	  *dst++ = *src++;
+	  *dst++ = src[i++];
 	}
     }
   *dst = 0;
 }
 
 void
-argcv_quote_copy (char *dst, const char *src)
+argcv_quote_copy_n (char *dst, const char *src, size_t size)
 {
-  for (; *src; src++)
+  const char *end = src + size;
+  for (; src < end; src++)
     {
       if (*src == '"')
 	{
 	  *dst++ = '\\';
-	  *dst++ = '"';
+	  *dst++ = *src;
 	}
-      else if (*src != '\t' && isprint(*src))
+      else if (*src != '\t' && *src != '\\' && isprint(*src))
 	*dst++ = *src;      
       else
 	{
@@ -301,11 +324,16 @@ argcv_quote_copy (char *dst, const char *src)
     }
 }
 
+void
+argcv_quote_copy (char *dst, const char *src)
+{
+	argcv_quote_copy_n (dst, src, strlen (src));
+}
+
 int
-argcv_get (const char *command, const char *delim, const char* cmnt,
-	   int *argc, char ***argv)
+argcv_get_n (const char *command, int len, const char *delim, const char *cmnt,
+	     int *argc, char ***argv)
 {
-  int len = strlen (command);
   int i = 0;
   int start, end, save;
 
@@ -315,35 +343,62 @@ argcv_get (const char *command, const char *delim, const char* cmnt,
   *argc = 0;
   save = 0;
 
+  if (!delim)
+    delim = "";
+  if (!cmnt)
+    cmnt = "";
+  
   while (argcv_scan (len, command, delim, cmnt, &start, &end, &save) <= len)
       (*argc)++;
 
   *argv = calloc ((*argc + 1), sizeof (char *));
-
+  if (*argv == NULL)
+    return ENOMEM;
+  
   i = 0;
   save = 0;
   for (i = 0; i < *argc; i++)
     {
       int n;
+      int unquote;
+      
       argcv_scan (len, command, delim, cmnt, &start, &end, &save);
 
       if ((command[start] == '"' || command[end] == '\'')
 	  && command[end] == command[start])
 	{
-	  start++;
-	  end--;
+	  if (start < end)
+	    {
+	      start++;
+	      end--;
+	    }
+	  unquote = 0;
 	}
+      else
+	unquote = 1;
+      
       n = end - start + 1;
       (*argv)[i] = calloc (n+1,  sizeof (char));
       if ((*argv)[i] == NULL)
-	return 1;
-      argcv_unquote_copy ((*argv)[i], &command[start], n);
+	return ENOMEM;
+      if (unquote)
+	argcv_unquote_copy ((*argv)[i], &command[start], n);
+      else
+	memcpy ((*argv)[i], &command[start], n);
       (*argv)[i][n] = 0;
     }
   (*argv)[i] = NULL;
   return 0;
 }
 
+int
+argcv_get (const char *command, const char *delim, const char *cmnt,
+	   int *argc, char ***argv)
+{
+  return argcv_get_n (command, strlen (command), delim, cmnt, argc, argv);
+}
+
+
 /*
  * frees all elements of an argv array
  * argc is the number of elements
@@ -356,7 +411,7 @@ argcv_free (int argc, char **argv)
     if (argv[argc])
       free (argv[argc]);
   free (argv);
-  return 1;
+  return 0;
 }
 
 /* Take a argv an make string separated by ' '.  */
@@ -369,16 +424,16 @@ argcv_string (int argc, char **argv, char **pstring)
 
   /* No need.  */
   if (pstring == NULL)
-    return 1;
+    return EINVAL;
 
   buffer = malloc (1);
   if (buffer == NULL)
-    return 1;
+    return ENOMEM;
   *buffer = '\0';
 
   for (len = i = j = 0; i < argc; i++)
     {
-      int quote = 0;
+      int quote;
       int toklen;
 
       toklen = argcv_quoted_length (argv[i], &quote);
@@ -389,7 +444,7 @@ argcv_string (int argc, char **argv, char **pstring)
       
       buffer = realloc (buffer, len);
       if (buffer == NULL)
-        return 1;
+        return ENOMEM;
 
       if (i != 0)
 	buffer[j++] = ' ';
@@ -409,21 +464,3 @@ argcv_string (int argc, char **argv, char **pstring)
   return 0;
 }
 
-#if 0
-char *command = "set prompt=\"& \a\\\"\" \\x25\\0145\\098\\ta";
-
-main(int xargc, char **xargv)
-{
-  int i, argc;
-  char **argv;
-  char *s;
-  
-  argcv_get (xargv[1] ? xargv[1]:command, "=", "#", &argc, &argv);
-  printf ("%d args:\n", argc);
-  for (i = 0; i < argc; i++)
-    printf ("%s\n", argv[i]);
-  printf ("===\n");
-  argcv_string (argc, argv, &s);
-  printf ("%s\n", s);
-}
-#endif
