@@ -1,5 +1,5 @@
 /* This file is part of GNU Radius.
-   Copyright (C) 2000,2001,2002,2003 Free Software Foundation
+   Copyright (C) 2000,2001,2002,2003,2004,2005,2007 Free Software Foundation
   
    GNU Radius is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -15,7 +15,6 @@
    along with GNU Radius; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
 
-#define RADIUS_MODULE_RADIUSD_C
 #ifdef HAVE_CONFIG_H
 # include <config.h>
 #endif
@@ -32,55 +31,66 @@
 #include <sys/wait.h>
 
 #include <radiusd.h>
-#include <radargp.h>
-#include <radutmp.h>
+#include <radius/radargp.h>
+#include <radius/radutmp.h>
+#include <radius/argcv.h>
 #include <rewrite.h>
-#include <argcv.h>
 #include <snmp/asn1.h>
 #include <snmp/snmp.h>
-#ifdef USE_SQL
-# include <radsql.h>
-#endif
 #include <timestr.h>
 
 const char *argp_program_version = "radiusd (" PACKAGE ") " VERSION;
 static char doc[] = N_("GNU radius daemon");
 
+#define SHOW_DEFAULTS_OPTION 256
+
 static struct argp_option options[] = {
+#define GRP 100	
         {NULL, 0, NULL, 0,
-         N_("radiusd specific switches:"), 0},
+         N_("radiusd specific switches:"), GRP},
+        {"foreground", 'f', NULL, 0,
+         N_("Stay in foreground"), GRP+1},
+        {"mode", 'm', "{t|c|b}", 0,
+         N_("Select operation mode: test, checkconf, builddbm."), GRP+1},
+        {"single-process", 's', NULL, 0,
+         N_("Run in single process mode"), GRP+1},
+        {"pid-file-dir", 'P', N_("DIR"), 0,
+         N_("Store pidfile in DIR"), GRP+1},
+	{"show-defaults", SHOW_DEFAULTS_OPTION, NULL, 0,
+	 N_("Show compilation defaults"), GRP+1},
+	{"quiet", 'q', NULL, 0,
+	 N_("Quiet mode (valid only with --mode)"), GRP+1},
+#undef GRP
+#define GRP 200
+	{NULL, 0, NULL, 0,
+         N_("Daemon configuration options. Please use raddb/config instead."),
+	 GRP},
+	
         {"log-auth-detail", 'A', 0, 0,
-         N_("Do detailed authentication logging"), 0},
+         N_("Do detailed authentication logging"), GRP+1},
         {"acct-directory",  'a', N_("DIR"), 0,
-         N_("Set accounting directory"), 0},
+         N_("Set accounting directory"), GRP+1},
 #ifdef USE_DBM
         {"dbm", 'b', NULL, 0,
-         N_("Enable DBM support"), 0},
+         N_("Enable DBM support"), GRP+1},
 #endif
-        {"foreground", 'f', NULL, 0,
-         N_("Stay in foreground"), 0},
         {"logging-directory", 'l', N_("DIR"), 0, 
-         N_("Set logging directory name"), 0},
-        {"mode", 'm', "{t|c|b}", 0,
-         N_("Select operation mode: test, checkconf, builddbm.")},
+         N_("Set logging directory name"), GRP+1},
         {"do-not-resolve", 'n', NULL, 0,
-         N_("Do not resolve IP addresses"), 0},
+         N_("Do not resolve IP addresses"), GRP+1},
         {"ip-address", 'i', N_("IPADDR"), 0,
-         N_("Listen on IPADDR"), 0},
+         N_("Listen on IPADDR"), GRP+1},
         {"port", 'p', "NUMBER", 0,
-         N_("Set authentication port number"), 0},
-        {"pid-file-dir", 'P', N_("DIR"), 0,
-         N_("Store pidfile in DIR"), 0},
+         N_("Set authentication port number"), GRP+1},
         {"log-stripped-names", 'S', NULL, 0,
-         N_("Strip prefixes/suffixes off user names before logging")},
-        {"single-process", 's', NULL, 0,
-         N_("Run in single process mode"), 0},
+         N_("Strip prefixes/suffixes off user names before logging"), GRP+1},
         {"debug", 'x', N_("DEBUGSPEC"), 0,
-         N_("Set debugging level"), 0},
+         N_("Set debugging level"), GRP+1},
         {"log-auth", 'y', NULL, 0,
-         N_("Log authentications"), 0},
+         N_("Log authentications"), GRP+1},
         {"log-auth-pass", 'z', NULL, 0,
-         N_("Log users' passwords"), 0},
+         N_("Log users' passwords"), GRP+1},
+#undef GRP
         {NULL, 0, NULL, 0, NULL, 0}
 };
 
@@ -89,6 +99,8 @@ static struct argp_option options[] = {
 
 int        debug_flag;     /* can be raised from debugger only */
 int        log_mode;       /* logging mode */
+int        console_logging_priority = -1; /* Default priority for console
+					     logging */
 char       *auth_log_hook; /* Authentication logging hook function */
 
 static int foreground; /* Stay in the foreground */
@@ -97,7 +109,10 @@ int spawn_flag;        /* Whether to spawn new children for handling
 
 int use_dbm = 0;       /* Use DBM storage */
 int auth_detail = 0;   /* Produce detailed logs of authentication packets */
+char *auth_detail_template; /* Template for filenames of these logs */
 int acct_detail = 1;   /* Produce detailed logs of accounting packets */
+char *acct_detail_template; /* Template for filenames of these logs */
+int acct_system = 1;   /* Run system accounting into radutmp/radwtmp files */
 int auth_trace_rules = 0; /* Produce trace logs for each auth request */
 int acct_trace_rules = 0; /* Produce trace logs for each acct request */
 int strip_names;          /* Strip preffixes/suffixes off the usernames */
@@ -140,11 +155,11 @@ unsigned process_timeout = PROCESS_TIMEOUT;
 unsigned radiusd_write_timeout = RADIUSD_WRITE_TIMEOUT;
 unsigned radiusd_read_timeout = RADIUSD_READ_TIMEOUT;
 
-UINT4 warning_seconds;
+grad_uint32_t warning_seconds;
 int use_guile;
 char *message_text[MSG_COUNT];
-UINT4 myip = INADDR_ANY;
-UINT4 ref_ip = INADDR_ANY;
+grad_uint32_t myip = INADDR_ANY;
+grad_uint32_t ref_ip = INADDR_ANY;
 int auth_port;
 int acct_port;
 
@@ -175,7 +190,7 @@ parse_opt(int key, char *arg, struct argp_state *state)
                 break;
 		
         case 'a':
-                radacct_dir = estrdup(optarg);
+                grad_acct_dir = grad_estrdup(arg);
                 break;
 		
 #ifdef USE_DBM
@@ -189,7 +204,7 @@ parse_opt(int key, char *arg, struct argp_state *state)
                 break;
 		
         case 'l':
-                radlog_dir = estrdup(optarg);
+                grad_log_dir = grad_estrdup(arg);
                 break;
 		
         case 'm':
@@ -219,24 +234,28 @@ parse_opt(int key, char *arg, struct argp_state *state)
                 break;
 		
         case 'n':
-                resolve_hostnames = 0;
+                grad_resolve_hostnames = 0;
                 break;
 		
         case 'i':
-                if ((myip = ip_gethostaddr(optarg)) == 0)
+                if ((myip = grad_ip_gethostaddr(arg)) == 0)
                         fprintf(stderr,
                                 _("invalid IP address: %s"),
-                                optarg);
+                                arg);
                 break;
 		
         case 'P':
-                radpid_dir = optarg;
+                grad_pid_dir = arg;
                 break;
 		
         case 'p':
-                auth_port = atoi(optarg);
+                auth_port = atoi(arg);
 		acct_port = auth_port+1;
                 break;
+
+	case 'q':
+		console_logging_priority = L_ERR;
+		break;
 		
         case 'S':
                 strip_names++;  
@@ -247,8 +266,8 @@ parse_opt(int key, char *arg, struct argp_state *state)
                 break;
 		
         case 'x':
-                x_debug_spec = optarg;
-                set_debug_levels(optarg);
+                x_debug_spec = arg;
+                grad_set_debug_levels(arg);
                 break;
 		
         case 'y':
@@ -258,6 +277,10 @@ parse_opt(int key, char *arg, struct argp_state *state)
         case 'z':
                 log_mode |= RLOG_AUTH_PASS;
                 break;
+
+	case SHOW_DEFAULTS_OPTION:
+		show_compilation_defaults();
+		exit(0);
 		
         default:
                 return ARGP_ERR_UNKNOWN;
@@ -270,7 +293,7 @@ static struct argp argp = {
         parse_opt,
         NULL,
         doc,
-        rad_common_argp_child,
+        grad_common_argp_child,
         NULL, NULL
 };
 
@@ -279,23 +302,23 @@ static struct argp argp = {
 void
 set_config_defaults()
 {
-        username_valid_chars = estrdup(".-_!@#$%^&\\/");
+        username_valid_chars = grad_estrdup(".-_!@#$%^&\\/");
         message_text[MSG_ACCOUNT_CLOSED] =
-                estrdup(_("Sorry, your account is currently closed\n"));
+                grad_estrdup(_("Sorry, your account is currently closed\n"));
         message_text[MSG_PASSWORD_EXPIRED] =
-                estrdup(_("Password has expired\n"));
+                grad_estrdup(_("Password has expired\n"));
         message_text[MSG_PASSWORD_EXPIRE_WARNING] =
-                estrdup(_("Password will expire in %R{Password-Expire-Days} Days\n"));
+                grad_estrdup(_("Password will expire in %R{Password-Expire-Days} Days\n"));
         message_text[MSG_ACCESS_DENIED] =
-                estrdup(_("\nAccess denied\n"));
+                grad_estrdup(_("\nAccess denied\n"));
         message_text[MSG_REALM_QUOTA] =
-                estrdup(_("\nRealm quota exceeded - access denied\n"));
+                grad_estrdup(_("\nRealm quota exceeded - access denied\n"));
         message_text[MSG_MULTIPLE_LOGIN] =
-                estrdup(_("\nYou are already logged in %R{Simultaneous-Use} times - access denied\n"));
+                grad_estrdup(_("\nYou are already logged in %R{Simultaneous-Use} times - access denied\n"));
         message_text[MSG_SECOND_LOGIN] =
-                estrdup(_("\nYou are already logged in - access denied\n"));
+                grad_estrdup(_("\nYou are already logged in - access denied\n"));
         message_text[MSG_TIMESPAN_VIOLATION] =
-                estrdup(_("You are calling outside your allowed timespan\n"));
+                grad_estrdup(_("You are calling outside your allowed timespan\n"));
 }
 
 static int
@@ -333,7 +356,7 @@ terminate_subprocesses()
 	request_init_queue();
 	
 	/* Terminate all subprocesses */
-	radlog(L_INFO, _("Terminating the subprocesses"));
+	grad_log(L_INFO, _("Terminating the subprocesses"));
 	rpp_kill(-1, SIGTERM);
 	
 	max_ttl(&t);
@@ -344,11 +367,11 @@ terminate_subprocesses()
 		if (time(NULL) >= t) {
 			if (kill_sent) {
 				int n = rpp_count();
-				radlog(L_CRIT,
-				       ngettext("%d process left!",
-						"%d processes left!",
-						n),
-				       n);
+				grad_log(L_CRIT,
+				         ngettext("%d process left!",
+					 	  "%d processes left!",
+						  n),
+				         n);
 				break;
 			}
 			max_ttl(&t);
@@ -370,22 +393,28 @@ radiusd_postconfig_hook(void *a ARG_UNUSED, void *b ARG_UNUSED)
 {
 	if (radius_mode == MODE_DAEMON && radius_count_channels() == 0) {
 		if (foreground) {
-			radlog(L_ALERT,
-			       _("Radiusd is not listening on any port."));
+			grad_log(L_ALERT,
+			         _("Radiusd is not listening on any port."));
 			exit(1);
 		} else
-			radlog(L_ALERT,
-			       _("Radiusd is not listening on any port. Trying to continue anyway..."));
+			grad_log(L_ALERT,
+			         _("Radiusd is not listening on any port. Trying to continue anyway..."));
 	}
 }
 
+static void
+daemon_postconfig_hook(void *a ARG_UNUSED, void *b ARG_UNUSED)
+{
+	system_acct_init();
+}
+
 void
 radiusd_setup()
 {
 	int i;
 
 	/* Close unneeded file descriptors */
-        for (i = getmaxfd(); i >= 3; i--)
+        for (i = grad_max_fd(); i >= 3; i--)
                 close(i);
         /* Determine default port numbers for authentication and accounting */
 	if (auth_port == 0) 
@@ -396,6 +425,19 @@ radiusd_setup()
 	snmp_port = get_port_number("snmp", "udp", 161);
 #endif
         srand(time(NULL));
+	
+#ifdef HAVE_CRYPT_SET_FORMAT
+	/* MD5 hashes are handled by libgnuradius function md5crypt(). To
+	   handle DES hashes it falls back to system crypt(). The behaviour
+	   of the latter on FreeBSD depends upon a 'default format'. so e.g.
+	   crypt() may generate MD5 hashes even if presented with a valid
+	   MD5 salt.
+	   
+	   To make sure this does not happen, we need to set the default
+	   crypt() format. */
+	
+	crypt_set_format("des");
+#endif
 
 	/* Register radiusd hooks first. This ensures they will be
 	   executed after all other hooks */
@@ -403,13 +445,16 @@ radiusd_setup()
 	radiusd_set_postconfig_hook(radiusd_postconfig_hook, NULL, 0);
 
 	rewrite_init();
-        snmp_init(0, 0, (snmp_alloc_t)emalloc, (snmp_free_t)efree);
+	dynload_init();
+        snmp_init(0, 0, (snmp_alloc_t)grad_emalloc, (snmp_free_t)grad_free);
+	mlc_init();
+	sql_init();
 }
 
 void
 common_init()
 {
-	radlog(L_INFO, _("Starting"));
+	grad_log(L_INFO, _("Starting"));
 
 	radiusd_pid = getpid();
 	radius_input = input_create();
@@ -431,8 +476,7 @@ common_init()
 #endif
 	acct_init();
 	radiusd_reconfigure();
-	radpath_init();
-	radlog(L_INFO, _("Ready"));
+	grad_log(L_INFO, _("Ready"));
 }
 
 
@@ -446,7 +490,7 @@ radiusd_daemon()
         
         switch (pid = fork()) {
         case -1:
-                radlog(L_CRIT|L_PERROR, "fork");
+                grad_log(L_CRIT|L_PERROR, "fork");
                 exit(1);
         case 0: /* Child */
                 break;
@@ -459,7 +503,7 @@ radiusd_daemon()
 #endif
         /* SIGHUP is ignored because when the session leader terminates
            all process in the session are sent the SIGHUP.  */
-        rad_set_signal(SIGHUP, SIG_IGN);
+        grad_set_signal(SIGHUP, SIG_IGN);
 
         /* fork() again so the parent, can exit. This means that we, as a
            non-session group leader, can never regain a controlling
@@ -468,15 +512,16 @@ radiusd_daemon()
         case 0:
                 break;
         case -1:
-                radlog(L_CRIT|L_PERROR, "fork");
+                grad_log(L_CRIT|L_PERROR, "fork");
                 exit(1);
         default:
                 exit(0);
         }
 
-        /* FIXME: This is needed for messages generated by guile
-           functions. */
-        p = mkfilename(radlog_dir, "radius.stderr");
+        /* This is needed for messages generated by guile
+           functions.
+	   FIXME: The compiled-in value of grad_log_dir is used */
+        p = grad_mkfilename(grad_log_dir, "radius.stderr");
         i = open(p, O_CREAT|O_WRONLY, 0644);
         if (i != -1) {
                 if (i != 2) 
@@ -487,9 +532,8 @@ radiusd_daemon()
                         close(i);
                 fflush(stdout);
                 fflush(stderr);
-        }
-	
-        efree(p);
+        } 	
+        grad_free(p);
 }
 
 int
@@ -523,16 +567,18 @@ radiusd_main()
 		if (myip != INADDR_ANY)
 			ref_ip = myip;
 		else
-			ref_ip = get_first_ip();
+			ref_ip = grad_first_ip();
 		if (ref_ip == INADDR_ANY)
-		    radlog(L_ALERT, _("can't find out my own IP address"));
+		    grad_log(L_ALERT, _("can't find out my own IP address"));
 		
 		chdir("/");
 		umask(022);
 
                 if (!foreground)
                         radiusd_daemon();
-		
+		/* Install daemon-specific hook */
+		radiusd_set_postconfig_hook(daemon_postconfig_hook,
+					    NULL, 0);
                 common_init();
         }
 
@@ -541,9 +587,9 @@ radiusd_main()
 	if (radiusd_user.username) {
 		char *p;
 		log_change_owner(&radiusd_user);
-		p = mkfilename(radlog_dir, "radius.stderr");
+		p = grad_mkfilename(grad_log_dir, "radius.stderr");
 		chown(p, radiusd_user.uid, radiusd_user.gid);
-		efree(p);
+		grad_free(p);
 		radius_switch_to_user(&radiusd_user);
 	}
 
@@ -567,12 +613,13 @@ main(int argc, char **argv)
            It means developer is taking control in his hands, so
            we won't modify any variables that could prevent him
            from doing so. */
-        if (debug_flag == 0) {
+	if (debug_flag == 0) {
                 foreground = 0;
                 spawn_flag = 1;
         }
-        app_setup();
-
+        grad_app_setup();
+	grad_set_logger(radiusd_logger);
+	
         /* save the invocation */
         xargc = argc;
         xargv = argv;
@@ -582,12 +629,12 @@ main(int argc, char **argv)
 
         /* Process the options.  */
         argp_program_version_hook = version;
-        if (rad_argp_parse(&argp, &argc, &argv, 0, NULL, NULL))
+        if (grad_argp_parse(&argp, &argc, &argv, 0, NULL, NULL))
                 return 1;
 
         log_set_default("default.log", -1, -1);
         if (radius_mode != MODE_DAEMON)
-                log_set_to_console();
+                log_set_to_console(-1, console_logging_priority);
 
 	radiusd_setup();
 	radiusd_start();
@@ -629,7 +676,7 @@ radiusd_suspend()
 {
 	if (suspend_flag == 0) {
 		terminate_subprocesses();
-		radlog(L_NOTICE, _("RADIUSD SUSPENDED"));
+		grad_log(L_NOTICE, _("RADIUSD SUSPENDED"));
 		suspend_flag = 1;
 	}
 }
@@ -659,7 +706,7 @@ check_reload()
 		break;
 		
         case CMD_RELOAD:
-                radlog(L_INFO, _("Reloading configuration now"));
+                grad_log(L_INFO, _("Reloading configuration now"));
                 radiusd_reconfigure();
                 break;
 		
@@ -671,7 +718,7 @@ check_reload()
                 break;
 		
         case CMD_DUMPDB:
-                radlog(L_INFO, _("Dumping users db to `%s'"),
+                grad_log(L_INFO, _("Dumping users db to `%s'"),
 		       RADIUS_DUMPDB_NAME);
                 dump_users_db();
                 break;
@@ -704,34 +751,34 @@ radiusd_close_channel(int fd)
 }
 
 void
-radiusd_cleanup()
+radiusd_collect_children()
 {
 	pid_t pid;
 	int status;
-	char buffer[128];
-	
-        for (;;) {
 
+	for (;;) {
 		pid = waitpid((pid_t)-1, &status, WNOHANG);
-                if (pid <= 0)
-                        break;
-
-		format_exit_status(buffer, sizeof buffer, status);
-		radlog(L_NOTICE, "child %d %s", pid, buffer);
-
-		rpp_remove(pid);
+		if (pid <= 0)
+			break;
+		rpp_status_changed(pid, status);
 	}
 }
 
+void
+radiusd_cleanup()
+{
+	rpp_collect_exited ();
+}
+
 void
 radiusd_restart()
 {
 	pid_t pid;
 	
-	radlog(L_NOTICE, _("restart initiated"));
+	grad_log(L_NOTICE, _("restart initiated"));
 	if (xargv[0][0] != '/') {
-		radlog(L_ERR,
-		       _("can't restart: not started as absolute pathname"));
+		grad_log(L_ERR,
+		         _("can't restart: not started as absolute pathname"));
 		return;
 	}
 
@@ -742,8 +789,8 @@ radiusd_restart()
 	else 
 		pid = fork();
 	if (pid < 0) {
-		radlog(L_CRIT|L_PERROR,
-		       _("radiusd_restart: cannot fork"));
+		grad_log(L_CRIT|L_PERROR,
+		         _("radiusd_restart: cannot fork"));
 		return;
 	}
 	
@@ -758,9 +805,9 @@ radiusd_restart()
 	sleep(10);
 
 	/* Child */
-	radlog(L_NOTICE, _("restarting radius"));
+	grad_log(L_NOTICE, _("restarting radius"));
 	execvp(xargv[0], xargv);
-	radlog(L_CRIT|L_PERROR, _("RADIUS NOT RESTARTED: exec failed"));
+	grad_log(L_CRIT|L_PERROR, _("RADIUS NOT RESTARTED: exec failed"));
 	exit(1);
 	/*NOTREACHED*/
 }
@@ -797,7 +844,7 @@ radiusd_exit()
 	radiusd_pidfile_remove(RADIUSD_PID_FILE);
 	
 	radiusd_flush_queue();
-	radlog(L_CRIT, _("Normal shutdown."));
+	grad_log(L_CRIT, _("Normal shutdown."));
 
 	rpp_kill(-1, SIGTERM);
 	radiusd_exit0();
@@ -806,16 +853,14 @@ radiusd_exit()
 void
 radiusd_exit0()
 {
-#ifdef USE_SQL
         radiusd_sql_shutdown();
-#endif
         exit(0);
 }
 
 void
 radiusd_main_loop()
 {
-        radlog(L_INFO, _("Ready to process requests."));
+        grad_log(L_INFO, _("Ready to process requests."));
 
         for (;;) {
 		log_open(L_MAIN);
@@ -833,35 +878,35 @@ struct hook_rec {
 	int once; /* Run once and remove */
 };
 
-static RAD_LIST /* of struct hook_rec */ *preconfig;
-static RAD_LIST /* of struct hook_rec */ *postconfig;
+static grad_list_t /* of struct hook_rec */ *preconfig;
+static grad_list_t /* of struct hook_rec */ *postconfig;
 
 void
 radiusd_set_preconfig_hook(void (*f)(void *, void *), void *p, int once)
 {
-	struct hook_rec *hp = emalloc(sizeof(*hp));
+	struct hook_rec *hp = grad_emalloc(sizeof(*hp));
 	hp->function = f;
 	hp->data = p;
 	hp->once = once;
 	if (!preconfig)
-		preconfig = list_create();
-	list_prepend(preconfig, hp);
+		preconfig = grad_list_create();
+	grad_list_prepend(preconfig, hp);
 }
 
 void
 radiusd_set_postconfig_hook(void (*f)(void *, void *), void *p, int once)
 {
-	struct hook_rec *hp = emalloc(sizeof(*hp));
+	struct hook_rec *hp = grad_emalloc(sizeof(*hp));
 	hp->function = f;
 	hp->data = p;
 	hp->once = once;
 	if (!postconfig)
-		postconfig = list_create();
-	list_prepend(postconfig, hp);
+		postconfig = grad_list_create();
+	grad_list_prepend(postconfig, hp);
 }
 
 struct hook_runtime_closure {
-	RAD_LIST *list;
+	grad_list_t *list;
 	void *call_data;
 };
 
@@ -872,8 +917,8 @@ _hook_call(void *item, void *data)
 	struct hook_runtime_closure *clos = data;
 	hp->function(hp->data, clos->call_data);
 	if (hp->once) {
-		list_remove(clos->list, hp, NULL);
-		efree(hp);
+		grad_list_remove(clos->list, hp, NULL);
+		grad_free(hp);
 	}
 	return 0;
 }
@@ -884,7 +929,7 @@ radiusd_run_preconfig_hooks(void *data)
 	struct hook_runtime_closure clos;
 	clos.list = preconfig;
 	clos.call_data = data;
-	list_iterate(clos.list, _hook_call, &clos);
+	grad_list_iterate(clos.list, _hook_call, &clos);
 }
 
 void
@@ -893,7 +938,7 @@ radiusd_run_postconfig_hooks(void *data)
 	struct hook_runtime_closure clos;
 	clos.list = postconfig;
 	clos.call_data = data;
-	list_iterate(clos.list, _hook_call, &clos);
+	grad_list_iterate(clos.list, _hook_call, &clos);
 }
 
 void
@@ -904,20 +949,21 @@ radiusd_reconfigure()
 
 	radiusd_run_preconfig_hooks(NULL);
 	
-	radlog(L_INFO, _("Loading configuration files."));
+	grad_log(L_INFO, _("Loading configuration files."));
 	/* Read main configuration file */
-        filename = mkfilename(radius_dir, RADIUS_CONFIG);
+        filename = grad_mkfilename(grad_config_dir, RADIUS_CONFIG);
         cfg_read(filename, config_syntax, NULL);
-	efree(filename);
+	grad_free(filename);
 
 	/* Read other files */
         rc = reload_config_file(reload_all);
         
         if (rc) {
-                radlog(L_CRIT, _("Errors reading config file - EXITING"));
+                grad_log(L_CRIT, _("Errors reading config file - EXITING"));
                 exit(1);
         }
 
+	grad_path_init();
 	radiusd_run_postconfig_hooks(NULL);
 }
 
@@ -941,6 +987,7 @@ sig_handler(int sig)
 		break;
 
 	case SIGCHLD:
+		radiusd_collect_children ();
 		daemon_command = CMD_CLEANUP;
 		break;
 
@@ -957,21 +1004,20 @@ sig_handler(int sig)
 	default:
 		abort();
 	}
-	rad_reset_signal(sig, sig_handler);
+	grad_reset_signal(sig, sig_handler);
 }
 
 void
 radiusd_signal_init(RETSIGTYPE (*hp)(int sig))
 {
 	static int signum[] = {
-		SIGHUP, SIGUSR1, SIGUSR2, SIGQUIT,
-		SIGTERM, SIGCHLD, SIGBUS, 
-		SIGFPE, SIGSEGV, SIGILL, SIGPIPE
+		SIGHUP, SIGUSR1, SIGUSR2, SIGCHLD, 
+		SIGTERM, SIGQUIT, SIGPIPE
 	};
 	int i;
 
 	for (i = 0; i < sizeof(signum)/sizeof(signum[0]); i++)
-		rad_set_signal(signum[i], hp);
+		grad_set_signal(signum[i], hp);
 }
 
 
@@ -981,36 +1027,36 @@ void
 radiusd_pidfile_write(char *name)
 {
         pid_t pid = getpid();
-        char *p = mkfilename(radpid_dir, name);
+        char *p = grad_mkfilename(grad_pid_dir, name);
 	FILE *fp = fopen(p, "w"); 
 	if (fp) {
                 fprintf(fp, "%lu\n", (u_long) pid);
                 fclose(fp);
         }
-        efree(p);
+        grad_free(p);
 }	
 
 pid_t
 radiusd_pidfile_read(char *name)
 {
 	unsigned long val;
-	char *p = mkfilename(radpid_dir, name);
+	char *p = grad_mkfilename(grad_pid_dir, name);
 	FILE *fp = fopen(p, "r");
 	if (!fp)
 		return -1;
 	if (fscanf(fp, "%lu", &val) != 1)
 		val = -1;
 	fclose(fp);
-	efree(p);
+	grad_free(p);
 	return (pid_t) val;
 }
 
 void
 radiusd_pidfile_remove(char *name)
 {
-	char *p = mkfilename(radpid_dir, name);
+	char *p = grad_mkfilename(grad_pid_dir, name);
 	unlink(p);
-	efree(p);
+	grad_free(p);
 }
 
 
@@ -1053,7 +1099,7 @@ int
 udp_input_close(int fd, void *data)
 {
 	close(fd);
-	efree(data);
+	grad_free(data);
 	return 0;
 }
 
@@ -1072,7 +1118,7 @@ udp_input_cmp(const void *a, const void *b)
 }
 
 int
-udp_open(int type, UINT4 ipaddr, int port, int nonblock)
+udp_open(int type, grad_uint32_t ipaddr, int port, int nonblock)
 {
 	int fd;
 	struct sockaddr_in s;
@@ -1082,30 +1128,30 @@ udp_open(int type, UINT4 ipaddr, int port, int nonblock)
         s.sin_addr.s_addr = htonl(ipaddr);
         s.sin_port = htons(port);
 	if (p = input_find_channel(radius_input, "udp", &s)) {
-		char buffer[DOTTED_QUAD_LEN];
-		radlog(L_ERR,
-		       _("socket %s:%d is already assigned for %s"),
-		       ip_iptostr(ipaddr, buffer),
-		       port,
-		       request_class[p->type].name);
+		char buffer[GRAD_IPV4_STRING_LENGTH];
+		grad_log(L_ERR,
+		         _("socket %s:%d is already assigned for %s"),
+		         grad_ip_iptostr(ipaddr, buffer),
+		         port,
+		         request_class[p->type].name);
 		return 1;
 	}
 
         fd = socket(PF_INET, SOCK_DGRAM, 0);
 	if (nonblock) 
-		set_nonblocking(fd);
+		grad_set_nonblocking(fd);
         if (fd < 0) {
-                radlog(L_CRIT|L_PERROR, "%s socket",
-		       request_class[type].name);
+                grad_log(L_CRIT|L_PERROR, "%s socket",
+		         request_class[type].name);
 		return 1;
         }
         if (bind(fd, (struct sockaddr*) &s, sizeof(s)) < 0) {
-                radlog(L_CRIT|L_PERROR, "%s bind", request_class[type].name);
+                grad_log(L_CRIT|L_PERROR, "%s bind", request_class[type].name);
 		close(fd);
 		return 1;
 	}
 
-	p = emalloc(sizeof(*p));
+	p = grad_emalloc(sizeof(*p));
 	p->type = type;
 	p->addr = s;
 	input_register_channel(radius_input, "udp", fd, p);
@@ -1149,19 +1195,30 @@ rad_cfg_listen_auth(int argc, cfg_value_t *argv,
 	}
 	
 	for (i = 1; i < argc; i++)  
-		if (argv[i].type != CFG_HOST) {
+		if (argv[i].type == CFG_NETWORK) {
+			if (argv[i].v.network.netmask != 0xffffffffL)
+				cfg_type_error(CFG_HOST);
+		} else if (argv[i].type != CFG_HOST) {
 			cfg_type_error(CFG_HOST);
 			errcnt++;
 		}
 	
 	if (errcnt == 0 && radius_mode == MODE_DAEMON) {
-		for (i = 1; i < argc; i++) 
-			if (udp_open(R_AUTH,
-				     argv[i].v.host.ipaddr,
-				     argv[i].v.host.port > 0 ?
-				     argv[i].v.host.port : auth_port,
-				     0))
+		for (i = 1; i < argc; i++) {
+			grad_uint32_t ip;
+			int port;
+
+			if (argv[i].type == CFG_NETWORK) {
+				ip = argv[i].v.network.ipaddr;
+				port = auth_port;
+			} else {
+				ip = argv[i].v.host.ipaddr;
+				port = argv[i].v.host.port;
+			} 
+			
+			if (udp_open(R_AUTH, ip, port, 0))
 				errcnt++;
+		}
 	}
 	if (errcnt == 0)
 		_opened_auth_sockets++;
@@ -1193,18 +1250,30 @@ rad_cfg_listen_acct(int argc, cfg_value_t *argv,
 	}
 	
 	for (i = 1; i < argc; i++)  
-		if (argv[i].type != CFG_HOST) {
+		if (argv[i].type == CFG_NETWORK) {
+			if (argv[i].v.network.netmask != 0xffffffffL)
+				cfg_type_error(CFG_HOST);
+		} else if (argv[i].type != CFG_HOST) {
 			cfg_type_error(CFG_HOST);
 			errcnt++;
 		}
 	
 	if (errcnt == 0 && radius_mode == MODE_DAEMON) {
-		for (i = 1; i < argc; i++) 
-			udp_open(R_ACCT,
-				 argv[i].v.host.ipaddr,
-				 argv[i].v.host.port > 0 ?
-				 argv[i].v.host.port : acct_port,
-				 0);
+		for (i = 1; i < argc; i++) {
+			grad_uint32_t ip;
+			int port;
+
+			if (argv[i].type == CFG_NETWORK) {
+				ip = argv[i].v.network.ipaddr;
+				port = acct_port;
+			} else {
+				ip = argv[i].v.host.ipaddr;
+				port = argv[i].v.host.port;
+			} 
+			
+			if (udp_open(R_ACCT, ip, port, 0))
+				errcnt++;
+		}
 	}
 	_opened_acct_sockets++;
 	return 0;
@@ -1237,9 +1306,9 @@ rad_cfg_user(int argc, cfg_value_t *argv,
 int
 option_stmt_end(void *block_data, void *handler_data)
 {
-	if (radiusd_user.username && radiusd_user.uid != 0) {
-		radlog(L_WARN, _("Ignoring exec-program-user"));
-		efree(exec_user.username);
+	if (exec_user.username && radiusd_user.uid != 0) {
+		grad_log(L_WARN, _("Ignoring exec-program-user"));
+		grad_free(exec_user.username);
 		exec_user.username = NULL;
 	} else if (exec_user.username == NULL)
 		radius_get_user_ids(&exec_user, "daemon");
@@ -1249,27 +1318,27 @@ option_stmt_end(void *block_data, void *handler_data)
 struct cfg_stmt option_stmt[] = {
 	{ "source-ip", CS_STMT, NULL, cfg_get_ipaddr, &myip,
 	  NULL, NULL },
-	{ "max-requests", CS_STMT, NULL, cfg_get_integer, &max_requests,
+	{ "max-requests", CS_STMT, NULL, cfg_get_size_t, &max_requests,
 	  NULL, NULL },
-	{ "max-threads", CS_STMT, NULL, cfg_get_integer, &max_children,
+	{ "max-threads", CS_STMT, NULL, cfg_get_size_t, &max_children,
 	  NULL, NULL },
-	{ "max-processes", CS_STMT, NULL, cfg_get_integer, &max_children,
+	{ "max-processes", CS_STMT, NULL, cfg_get_size_t, &max_children,
 	  NULL, NULL },
-	{ "process-idle-timeout", CS_STMT, NULL, cfg_get_integer, &process_timeout,
+	{ "process-idle-timeout", CS_STMT, NULL, cfg_get_unsigned, &process_timeout,
 	  NULL, NULL },
 	{ "master-read-timeout", CS_STMT, NULL,
-	  cfg_get_integer, &radiusd_read_timeout, NULL, NULL },
+	  cfg_get_unsigned, &radiusd_read_timeout, NULL, NULL },
 	{ "master-write-timeout", CS_STMT, NULL,
-	  cfg_get_integer, &radiusd_write_timeout, NULL, NULL },
+	  cfg_get_unsigned, &radiusd_write_timeout, NULL, NULL },
 	{ "exec-program-user", CS_STMT, NULL, rad_cfg_user,
 	  &exec_user, NULL, NULL },
 	{ "radiusd-user", CS_STMT, NULL, rad_cfg_user,
 	  &radiusd_user, NULL, NULL },
-	{ "log-dir", CS_STMT, NULL, cfg_get_string, &radlog_dir,
+	{ "log-dir", CS_STMT, NULL, cfg_get_string, &grad_log_dir,
 	  NULL, NULL },
-	{ "acct-dir", CS_STMT, NULL, cfg_get_string, &radacct_dir,
+	{ "acct-dir", CS_STMT, NULL, cfg_get_string, &grad_acct_dir,
 	  NULL, NULL },
-	{ "resolve", CS_STMT, NULL, cfg_get_boolean, &resolve_hostnames,
+	{ "resolve", CS_STMT, NULL, cfg_get_boolean, &grad_resolve_hostnames,
 	  NULL, NULL },
 	{ "username-chars", CS_STMT, NULL, cfg_get_string,
 	  &username_valid_chars, NULL, NULL },
@@ -1318,6 +1387,8 @@ struct cfg_stmt auth_stmt[] = {
 	  cfg_get_integer, &request_class[R_AUTH].cleanup_delay, NULL, NULL },
 	{ "detail", CS_STMT, NULL, cfg_get_boolean, &auth_detail,
 	  NULL, NULL },
+	{ "detail-file-name", CS_STMT, NULL, cfg_get_string,
+	  &auth_detail_template, NULL, NULL },
 	{ "strip-names", CS_STMT, NULL, cfg_get_boolean, &strip_names,
 	  NULL, NULL },
 	{ "checkrad-assume-logged", CS_STMT, NULL,
@@ -1327,7 +1398,7 @@ struct cfg_stmt auth_stmt[] = {
 	  cfg_get_boolean, &auth_reject_malformed_names,
 	  NULL, NULL },
 	{ "password-expire-warning", CS_STMT, NULL,
-	  cfg_get_integer, &warning_seconds,
+	  cfg_get_uint32_t, &warning_seconds,
 	  NULL, NULL },
 	{ "compare-attribute-flag", CS_STMT, NULL,
 	  cfg_get_integer, &auth_comp_flag,
@@ -1338,11 +1409,11 @@ struct cfg_stmt auth_stmt[] = {
 	{ "spawn", CS_STMT, NULL, cfg_obsolete, NULL, NULL, NULL },
 	{ NULL, }
 };
-	
+
 struct cfg_stmt acct_stmt[] = {
 	{ "port", CS_STMT, NULL, cfg_get_port, &acct_port, NULL, NULL },
 	{ "listen", CS_STMT, NULL, rad_cfg_listen_acct, NULL, NULL, NULL },
-	{ "forward", CS_STMT, NULL, rad_cfg_forward_auth, NULL, NULL, NULL },
+	{ "forward", CS_STMT, NULL, rad_cfg_forward_acct, NULL, NULL, NULL },
 	{ "max-requests", CS_STMT, NULL,
 	  cfg_get_integer, &request_class[R_ACCT].max_requests,
 	  NULL, NULL },
@@ -1354,6 +1425,10 @@ struct cfg_stmt acct_stmt[] = {
 	  NULL, NULL },
 	{ "detail", CS_STMT, NULL, cfg_get_boolean, &acct_detail,
 	  NULL, NULL },
+	{ "detail-file-name", CS_STMT, NULL, cfg_get_string,
+	  &acct_detail_template, NULL, NULL },
+	{ "system", CS_STMT, NULL, cfg_get_boolean, &acct_system,
+	  NULL, NULL },
 	{ "compare-attribute-flag", CS_STMT, NULL,
 	  cfg_get_integer, &acct_comp_flag,
 	  NULL, NULL },
@@ -1378,10 +1453,13 @@ struct cfg_stmt config_syntax[] = {
 	  logging_stmt, logging_stmt_end },
 	{ "auth", CS_BLOCK, auth_stmt_begin, NULL, NULL, auth_stmt, NULL },
 	{ "acct", CS_BLOCK, acct_stmt_begin, NULL, NULL, acct_stmt, NULL  },
+	{ "mlc",  CS_BLOCK, NULL, NULL, NULL, mlc_stmt, NULL },
 	{ "proxy", CS_BLOCK, NULL, NULL, NULL, proxy_stmt, NULL  },
 	{ "rewrite", CS_BLOCK, rewrite_stmt_term, NULL, NULL, rewrite_stmt, NULL },
 	{ "filters", CS_BLOCK, filters_stmt_term, NULL, NULL, filters_stmt,
 	  NULL },
+	{ "loadable-modules", CS_BLOCK, dynload_stmt_term, NULL, NULL,
+	  dynload_stmt, NULL },
 #ifdef USE_DBM
 	{ "usedbm", CS_STMT, NULL, cfg_get_boolean, &use_dbm, NULL, NULL },
 #endif
