@@ -1,6 +1,6 @@
 /* This file is part of GNU Radius.
    Copyright (C) 2000,2001,2002,2003,2004,2005,
-   2006 Free Software Foundation, Inc.
+   2006,2007 Free Software Foundation, Inc.
 
    Written by Sergey Poznyakoff
 
@@ -117,6 +117,8 @@ shmem_free()
 {
 	munmap(shmem_base, shmem_size);
 	close(tempfd);
+	tempfd = -1;
+	offset = 0;
 }
 
 void *
@@ -188,8 +190,10 @@ stat_init()
 void
 stat_done()
 {
-	if (server_stat) 
+	if (server_stat) {
 		shmem_free();
+		server_stat = NULL;
+	}
 }
 
 #define FOR_EACH_PORT(p) \
@@ -231,7 +235,7 @@ stat_find_port(grad_nas_t *nas, int port_no)
 	port->ip = nas->netdef.ipaddr;
 	port->port_no = port_no;
 	
-	GRAD_DEBUG(1, ("next offset %d", port - port_stat));
+	GRAD_DEBUG1(1, "next offset %d", port - port_stat);
 
 	return port;
 }
@@ -355,12 +359,12 @@ stat_update(struct radutmp *ut, int status)
 		break;
 	}
 
-	GRAD_DEBUG(1,
-		("NAS %#x port %d: act %d, cnt %d, start %d, inuse %d/%d idle %d/%d",
-		 port->ip, port->port_no, port->active,
-		 port->count, port->start,
-		 port->inuse, port->maxinuse.time,
-		 port->idle, port->maxidle.time));
+	GRAD_DEBUG9(1,
+		"NAS %#x port %d: act %d, cnt %d, start %d, inuse %d/%d idle %d/%d",
+		port->ip, port->port_no, port->active,
+		port->count, port->start,
+		port->inuse, port->maxinuse.time,
+		port->idle, port->maxidle.time);
 }
 
 void
