@@ -23,9 +23,9 @@
 (define-module (gnuradius)
   :use-module (ice-9 format))
 
-(let ((lib-path "/home/gray/inst/lib/"))
+(let ((lib-path "/home/gray/alpha/lib/"))
   (load-extension (string-append
-		   lib-path "libguile-gnuradius-v-1.3.91") "radscm_init"))
+		   lib-path "libguile-gnuradius-v-1.4.1") "radscm_init"))
 
 ;; Exports go here
 (export %raddb-path
